@@ -103,7 +103,7 @@ static void icmp_send_probe (probe *pb, int ttl) {
 	    icmp6->icmp6_id = htons (ident);
 	    icmp6->icmp6_seq = htons(seq);
 
-	    icmp6->icmp6_cksum = in_csum (data, data_len);
+	    /*  icmp6->icmp6_cksum always computed by kernel internally   */
 	}
 
 
