@@ -130,7 +130,7 @@ static int check_sysctl (const char *name) {
 }
 
 
-static int tcp_init (const char *host, const sockaddr_any *dest,
+static int tcp_init (const sockaddr_any *dest,
 				unsigned int port_seq, size_t packet_len) {
 	int af = dest->sa.sa_family;
 	sockaddr_any src;
@@ -358,13 +358,10 @@ static void tcp_send_probe (probe *pb, int ttl) {
 
 	pb->send_time = get_time ();
 
-	if (raw_can_connect ()) {
-	    if (send (raw_sk, th, th->doff << 2, 0) < 0)
-		    error ("send");
-	} else {
-	    if (sendto (raw_sk, th, th->doff << 2, 0,
-				    &dest_addr.sa, sizeof (dest_addr)) < 0
-	    )  error ("sendto");
+	if (do_send (raw_sk, th, th->doff << 2, &dest_addr) < 0) {
+	    close (sk);
+	    pb->send_time = 0;
+	    return;
 	}
 
 
@@ -376,13 +373,12 @@ static void tcp_send_probe (probe *pb, int ttl) {
 }
 
 
-static void tcp_recv_probe (int sk, int revents, probe *probes,
-						unsigned int num_probes) {
+static void tcp_recv_probe (int sk, int revents) {
 	int af = dest_addr.sa.sa_family;
 	struct msghdr msg;
 	sockaddr_any from;
 	struct iovec iov;
-	int i, n, err;
+	int n, err;
 	probe *pb;
 	char buf[1024];
 	char control[1024];
@@ -439,9 +435,8 @@ static void tcp_recv_probe (int sk, int revents, probe *probes,
 	if (!equal_addr (&dest_addr, &from))
 		return;
 
-	for (i = 0; i < num_probes && probes[i].seq != sport; i++) ;
-	if (i >= num_probes)  return;
-	pb = &probes[i];
+	pb = probe_by_seq (sport);
+	if (!pb)  return;
 
 
 	parse_cmsg (pb, &msg);	    /*   err (if any), tstamp, ttl   */
