@@ -44,6 +44,14 @@
 #define IPV6_RECVHOPLIMIT IPV6_HOPLIMIT
 #endif
 
+#ifndef IP_PMTUDISC_PROBE
+#define IP_PMTUDISC_PROBE 3
+#endif
+
+#ifndef IPV6_PMTUDISC_PROBE
+#define IPV6_PMTUDISC_PROBE 3
+#endif
+
 
 #define MAX_HOPS	255
 #define MAX_PROBES	10
@@ -952,9 +960,11 @@ void tune_socket (int sk) {
 
 	if (af == AF_INET) {
 
-	    i = dontfrag ? IP_PMTUDISC_DO : IP_PMTUDISC_DONT;
-	    if (setsockopt (sk, SOL_IP, IP_MTU_DISCOVER, &i, sizeof(i)) < 0)
-		    error ("setsockopt IP_MTU_DISCOVER");
+	    i = dontfrag ? IP_PMTUDISC_PROBE : IP_PMTUDISC_DONT;
+	    if (setsockopt (sk, SOL_IP, IP_MTU_DISCOVER, &i, sizeof(i)) < 0 &&
+		(!dontfrag || (i = IP_PMTUDISC_DO,
+		 setsockopt (sk, SOL_IP, IP_MTU_DISCOVER, &i, sizeof(i)) < 0))
+	    )  error ("setsockopt IP_MTU_DISCOVER");
 
 	    if (tos) {
 		i = tos;
@@ -965,9 +975,12 @@ void tune_socket (int sk) {
 	}
 	else if (af == AF_INET6) {
 
-	    i = dontfrag ? IPV6_PMTUDISC_DO : IPV6_PMTUDISC_DONT;
-	    if (setsockopt (sk, SOL_IPV6, IPV6_MTU_DISCOVER, &i, sizeof(i)) < 0)
-		    error ("setsockopt IPV6_MTU_DISCOVER");
+	    i = dontfrag ? IPV6_PMTUDISC_PROBE : IPV6_PMTUDISC_DONT;
+	    if (setsockopt (sk, SOL_IPV6, IPV6_MTU_DISCOVER,&i,sizeof(i)) < 0 &&
+		(!dontfrag || (i = IPV6_PMTUDISC_DO,
+		 setsockopt (sk, SOL_IPV6, IPV6_MTU_DISCOVER,&i,sizeof(i)) < 0))
+	    )  error ("setsockopt IPV6_MTU_DISCOVER");
+
 
 	    if (flow_label) {
 		struct in6_flowlabel_req flr;
@@ -1286,7 +1299,7 @@ int do_send (int sk, const void *data, size_t len, const sockaddr_any *addr) {
 	    res = sendto (sk, data, len, 0, &addr->sa, sizeof (*addr));
 
 	if (res < 0 &&
-	    errno != ENOBUFS && errno != EAGAIN
+	    errno != ENOBUFS && errno != EAGAIN && errno != EMSGSIZE
 	)  error ("send");	/*  not recoverable   */
 
 	return res;
